/*

NSAttributedString-NIRAdditions.m

DirectionService - controll writting direction
Copyright (C) 2003 Nir Soffer

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details: http://www.gnu.org/licenses/gpl.html

*/

#import "NSAttributedString-NIRAdditions.h"
#import "NSString-NIRAdditions.h"


@implementation NSAttributedString (NSAttributedStringNIRAdditions)

//
// Shortcuts
//

- (NSRange)range; {
    // Return the full range of the reciver
    // A shortcut instead of writing this again and again
    return NSMakeRange(0, [self length]);
}

- (NSData *)RTF; {
    // Invoke -RTFFromRange:documentAttributes: with default options
    return [self RTFFromRange:[self range] documentAttributes:nil];
}
- (NSData *)RTFD; {
    // Invoke -RTFDFromRange:documentAttributes: with default options
    return [self RTFDFromRange:[self range] documentAttributes:nil];
}

@end


@implementation NSMutableAttributedString (NSMutableAttributedStringNIRAdditions)

- (void)encloseWithStart:(NSString *)start
                     end:(NSString *)end
           fixAttributes:(BOOL)fixAttributes
{
    // Enclose inside strings

    NSMutableString *string = [self mutableString];

    [string insertString:start atIndex:0];
    [string insertString:end atIndex:[self length]];

    if (fixAttributes) {
        [self fixAttributesInRange:NSMakeRange(0, [self length])];
    }
}

- (void)prefixLinesWithString:(NSString *)prefix
                fixAttributes:(BOOL)fixAttributes
{
    // Insert prefix string at each line start

    unsigned lineStart;
    unsigned location = 0;
    NSMutableString *string = [self mutableString];

    while (location < [string length])
    {
        // get line start
        [string getLineStart:&lineStart end:&location contentsEnd:nil
                    forRange:NSMakeRange(location, 1)];

        // Insert string
        [string insertString:prefix atIndex:lineStart];

        if (fixAttributes) {
            // Copy the attributes of the *next* character after the string.
            // The string get the previous character attributes when inserted,
            // which brakes the paragraph style of the line
            unsigned index = lineStart + [prefix length];
            NSDictionary *attributes = [self attributesAtIndex:index
                                                effectiveRange:nil];
            NSRange range = NSMakeRange(lineStart, [prefix length]);
            [self setAttributes:attributes range:range];
        }

        location = location + [prefix length]; // because we added characters
    }

    if (fixAttributes) {
        [self fixAttributesInRange:NSMakeRange(0, [self length])];
    }
}

- (unsigned)deleteCharactersFromSet:(NSCharacterSet *)set
                  fixAttributes:(BOOL)fixAttributes
{
    unsigned deleted;
    deleted = [[self mutableString] deleteCharactersFromSet:set];
    if (fixAttributes) {
        [self fixAttributesInRange:NSMakeRange(0, [self length])];
    }
    return deleted;
}

@end
