//
//  NSString-FriBidiAdditions.m
//  Hebrew Service
//
//  Created by Nir on 19/1/2006.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import "NSString-FriBidiAdditions.h"
#import "fribidi.h"


@implementation NSString (FriBidiAdditions)

- (NSString *)stringByOrderingVisually;
{
    NSString *result = nil; // failure by default
    unsigned int stringLength = [self length];
    FriBidiCharType baseDirection = FRIBIDI_TYPE_ON; // Natural order
    
    // Bytes buffers hold the string using either UCS4 or UTF8 encoding
    // both up to 4 bytes per character.
    // TODO: consider reusing a static buffer instead of allocating and 
    // freeing on each call.
    
    size_t bufferSize = (stringLength + 1) * 4;
    char *a = NULL, *b = NULL;
    
    a = malloc(bufferSize);
    if (a == NULL)
        goto cleanup;
    
    b = malloc(bufferSize);
    if (b == NULL)
        goto cleanup;
    
    // Encode, reorder and decode back. The functions here get two buffers 
    // and copy from one buffer to the other. Each call use the ouput buffer 
    // of the previous call, which save extra memory allocation.
    // a -> b, b -> a, a -> b...
    
    if (! CFStringGetCString((CFStringRef)self, a, bufferSize,
                             kCFStringEncodingUTF8))
        goto cleanup;
    
    fribidi_utf8_to_unicode(a, strlen(a), (FriBidiChar *)b);
    
    if (! fribidi_log2vis((FriBidiChar *)b, stringLength, &baseDirection, 
                          (FriBidiChar *)a, NULL, NULL, NULL))
        goto cleanup;
    
    fribidi_unicode_to_utf8((FriBidiChar *)a, stringLength, b);
    
    result = (NSString *)CFStringCreateWithCString(NULL, b, 
                                                   kCFStringEncodingUTF8);
    [result autorelease];
    
cleanup:
    free(a);
    free(b);
    
    return result;
}

@end
