/*

DirectionServiceTest.m

DirectionService - controll writting direction
Copyright (C) 2003 Nir Soffer

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details: http://www.gnu.org/licenses/gpl.html

*/

#import <CoreFoundation/CoreFoundation.h>
#import "NIRHebrewServiceTest.h"
#import "NIRHebrewService.h"
#import "NSString-NIRAdditions.h"
#import "NSAttributedString-NIRAdditions.h"
#import "NSPasteboard-NIRAdditions.h"

// Make Python like unicode string literals
static NSString *U(const char *s)
{
    return [NSString stringWithUTF8String:s];
}


@implementation NIRHebrewServiceTest

- (void)setUp
{
    service = [[NIRHebrewService alloc] init];
    pboard = [NSPasteboard pasteboardWithUniqueName];
}

- (void)tearDown
{
    [service release];
}

- (void)testConvertFromHebrewWindows
{
    // Converted text should from cp1255.txt should match text in
    // Unidoce.txt

    // Read file cp1255 string, interept as iso8859-1 (latin 1)
    NSString *cp1255 = [self stringFromTestFile:@"cp1255.txt"
                                       encoding:NSISOLatin1StringEncoding];
    // Read result file
    NSString *unicode = [self stringFromTestFile:@"unicode.txt"];

    // Convert string through the service
    [self assertTrue:[pboard putString:cp1255]
             message:@"Could not put on pboard"];
    NSString *error;
    [service convertFromHebrewWindows:pboard userData:nil error:&error];

    // Should get back unicode
    NSArray *types = [NSArray arrayWithObject:NSStringPboardType];
    NSString *type = [pboard availableTypeFromArray:types];
    [self assertNotNil:type message:@"pboard does not have a string"];
    NSString *converted = [pboard stringForType:type];
    [self assert:converted equals:unicode message:@"Strings do not match"];
}


- (void)testReverseSelection
{
    // Reverse selected text using Bidi algorithm
    NSString *test = U("שלום - hello");
    
    // Reverse string through the service
    [self assertTrue:[pboard putString:test]
             message:@"Could not put on pboard"];
    NSString *error;
    [service reverseSelection:pboard userData:nil error:&error];
    NSArray *types = [NSArray arrayWithObject:NSStringPboardType];
    NSString *type = [pboard availableTypeFromArray:types];
    
    [self assert:[pboard stringForType:type] equals:U("hello - םולש")];
}


//
// Utilities
//

- (NSString *)stringFromTestFile:(NSString *)file
                        encoding:(unsigned)encoding; {
    // Read file from disk, interept as text in encoding

    // Creates full path
    NSFileManager *fm = [NSFileManager defaultManager];
    NSString *currentDirectory = [fm currentDirectoryPath];
    NSString *path = [currentDirectory stringByDeletingLastPathComponent];
    path = [path stringByAppendingPathComponent:@"Test Files"];
    path = [path stringByAppendingPathComponent:file];
    return [NSString stringWithContentsOfFile:path
                                     encoding:encoding];    
}

- (NSString *)stringFromTestFile:(NSString *)file; {
    // invoke stringFromTestFile:encoding: with Unicode encoding
    return [self stringFromTestFile:file encoding:NSUnicodeStringEncoding];
}

@end
