/*

NSString-NIRAdditionsTest.m

DirectionService - controll writting direction
Copyright (C) 2003 Nir Soffer

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details: http://www.gnu.org/licenses/gpl.html

*/

#import "NSString-NIRAdditionsTest.h"
#import "NSString-NIRAdditions.h"

@implementation NSStringNIRAdditionsTest

- (void)setUp
{
    mString = [[NSMutableString alloc] init];
}

- (void)tearDown
{
    [mString release];
}

//
// NSString Tests
//

- (void)testStringWithCharacter
    // test that we do get a corect unichar
{
    unichar testUnichar = 0x202B;
    string = [NSString stringWithCharacter:testUnichar];

    // unichar should be one characters length containing the test unichar
    [self assertInt:[string length] equals:1];
    [self assertInt:[string characterAtIndex:0] equals:testUnichar];
}

- (void)testHasLines
{
    // characters
    [self assertFalse:[@"" hasLines]
              message:@"Empty string failed"];
    [self assertFalse:[@"First" hasLines]
              message:@"Some characters failed"];

    // paragraphs
    [self assertTrue:[@"\n" hasLines]
             message:@"Empty paragraph failed"];
    [self assertTrue:[@"First\n" hasLines]
             message:@"One paragraph failed"];
    [self assertTrue:[@"First\nSecond" hasLines]
             message:@"Multi paragraph failed"];
}

- (void)testStringByReversingLiterally {
    // Reverse string with no composed characters
    NSString *reversed;

    [self assert:@"" equals:[@"" stringByReversingLiterally:YES]
         message:@"Reversing empty string literally failed"];

    [self assert:@"1" equals:[@"1" stringByReversingLiterally:YES]
         message:@"Reversing single character string literally failed"];

    string 		= @"2003-11-09";
    reversed 	= @"90-11-3002";
    [self assert:reversed equals:[string stringByReversingLiterally:YES]
         message:@"Reversing literally failed"];

    [self assert:reversed equals:[string stringByReversingLiterally:NO]
         message:@"Reversing non-literally failed"];
}


//
// NSMutableString Tests
//

- (void)testDeleteCharactersFromSet
// Todo: build a bigger, more genral test?
{
    NSString *cleanString = @"This is just a test string\nA test String it is\n";
    NSCharacterSet *testSet = [NSCharacterSet characterSetWithRange:
        NSMakeRange(0x202A,3)];

    // clean string should return the same
    [mString setString:cleanString];
    [mString deleteCharactersFromSet:testSet];
    [self assert:mString equals:cleanString];

    // dirty string should return clean
    [mString insertString:[NSString stringWithCharacter:0x202A]
                  atIndex:0];
    [mString insertString:[NSString stringWithCharacter:0x202B]
                  atIndex:2];
    [mString insertString:[NSString stringWithCharacter:0x202C]
                  atIndex:[string length]];
    [mString deleteCharactersFromSet:testSet];
    [self assert:mString equals:cleanString];
}


//
// Utitiles
//

- (NSString *)stringFromTestFile:(NSString *)file
                        encoding:(unsigned)encoding; {
    // Read file from disk, interept as text in encoding

    // Creates full path
    NSFileManager *fm = [NSFileManager defaultManager];
    NSString *currentDirectory = [fm currentDirectoryPath];
    NSString *path = [currentDirectory stringByDeletingLastPathComponent];
    path = [path stringByAppendingPathComponent:@"Test Files"];
    path = [path stringByAppendingPathComponent:file];
    return [NSString stringWithContentsOfFile:path
                                     encoding:encoding];
}

- (NSString *)stringFromTestFile:(NSString *)file; {
    // invoke stringFromTestFile:encoding: with Unicode encoding
    return [self stringFromTestFile:file encoding:NSUnicodeStringEncoding];
}

@end
